@@ -2,16 +2,22 @@ package DateTime::Format::Natural;
 
 use strict;
 use warnings;
-use base qw(DateTime::Format::Natural::Base);
+use base qw(
+    DateTime::Format::Natural::Base
+    DateTime::Format::Natural::Duration
+    DateTime::Format::Natural::Formatted
+    DateTime::Format::Natural::Helpers
+);
 use boolean qw(true false);
 
 use Carp qw(croak);
 use DateTime ();
-use Date::Calc qw(Day_of_Week check_date);
 use List::MoreUtils qw(all any);
 use Params::Validate ':all';
+use Scalar::Util qw(blessed);
+use Storable qw(dclone);
 
-our $VERSION = '0.74';
+our $VERSION = '0.83';
 
 validation_options(
     on_fail => sub
@@ -37,13 +43,24 @@ sub new
 
 sub _init
 {
-    my ($self, %opts) = @_;
+    my $self = shift;
+    my %opts = @_;
 
-    $self->{Format}        = $opts{format}        || 'd/m/y';
-    $self->{Lang}          = $opts{lang}          || 'en';
-    $self->{Prefer_future} = $opts{prefer_future} || false;
-    $self->{Time_zone}     = $opts{time_zone}     || 'floating';
-    $self->{Opts}{daytime} = $opts{daytime};
+    my %presets = (
+        lang          => 'en',
+        format        => 'd/m/y',
+        prefer_future => false,
+        time_zone     => 'floating',
+    );
+    foreach my $opt (keys %presets) {
+        $self->{ucfirst $opt} = $presets{$opt};
+    }
+    foreach my $opt (keys %opts) {
+        if (defined $opts{$opt}) {
+            $self->{ucfirst $opt} = $opts{$opt};
+        }
+    }
+    $self->{Daytime} = $opts{daytime} || {};
 
     my $mod = __PACKAGE__.'::Lang::'.uc($self->{Lang});
     eval "use $mod"; die $@ if $@;
@@ -81,12 +98,23 @@ sub _init_check
                     eval { DateTime::TimeZone->new(name => shift) };
                     !$@;
                 }
-            }
+            },
         },
         daytime => {
             type => HASHREF,
             optional => true,
         },
+        datetime => {
+            type => OBJECT,
+            optional => true,
+            callbacks => {
+                'valid object' => sub
+                {
+                    my $obj = shift;
+                    blessed($obj) && $obj->isa('DateTime');
+                }
+            },
+        },
     });
 }
 
@@ -94,8 +122,9 @@ sub _init_vars
 {
     my $self = shift;
 
-    delete $self->{modified};
-    delete $self->{postprocess};
+    foreach my $member (qw(modified postprocess)) {
+        delete $self->{$member};
+    }
 }
 
 sub parse_datetime
@@ -104,111 +133,94 @@ sub parse_datetime
 
     $self->_parse_init(@_);
 
+    $self->{Input_string} = $self->{Date_string};
+
     my $date_string = $self->{Date_string};
     $date_string =~ tr/,//d;
 
-    my @count = $date_string =~ m![-./]!g;
-    my %count; $count{$_}++ foreach @count;
+    my ($formatted) = $date_string =~ m!^((?:\d+?[-./])+ (?:\d+?)) \b!x;
+
+    my %count;
+    if (defined $formatted) {
+        my @count = $formatted =~ m![-./]!g;
+        $count{$_}++ foreach @count;
+    }
+
+    $self->{tokens} = [];
+    $self->{traces} = [];
 
     if (scalar keys %count == 1 && $count{(keys %count)[0]} == 2) {
-        if ($date_string =~ /^\S+\b\s+\b\S+/) {
-            ($date_string, @{$self->{tokens}}) = split /\s+/, $date_string;
-            $self->{count}{tokens} = 1 + scalar @{$self->{tokens}};
-        }
-        else {
-            $self->{count}{tokens} = 1;
-        }
+        my $dt = $self->_parse_formatted_ymd($date_string, \%count);
+        return $dt if blessed($dt);
+    }
+    elsif (scalar keys %count == 1 && $count{(keys %count)[0]} == 1 && (keys %count)[0] eq '/') {
+        my $dt = $self->_parse_formatted_md($date_string);
+        return $dt if blessed($dt);
 
-        my $separator = quotemeta((keys %count)[0]);
-        my @chunks = split /$separator/, $date_string;
-
-        my $i = 0;
-        my %length = map { length $_ => $i++ } @chunks;
-
-        my $format = $self->{Format};
-
-        if (exists $length{4}) {
-            $format = join $separator, ($length{4} == 0 ? qw(yyyy mm dd) : qw(dd mm yyyy));
-        }
-        else {
-            $separator = do { local $_ = $format;
-                              tr/a-zA-Z//d;
-                              tr/a-zA-Z//cs;
-                              quotemeta; };
-        }
-
-        my @separated_order = split /$separator/, $format;
-        my $separated_index = 0;
-
-        my $separated_indices = { map { substr($_, 0, 1) => $separated_index++ } @separated_order };
-
-        my @bits = split $separator, $date_string;
-
-        my $century = $self->{datetime}
-                    ? int($self->{datetime}->year / 100)
-                    : substr((localtime)[5] + 1900, 0, 2);
-
-        my ($day, $month, $year) = map { $bits[$separated_indices->{$_}] } qw(d m y);
-
-        if (not defined $day && defined $month && defined $year) {
-            $self->_set_failure;
-            $self->_set_error("('format' parameter invalid)");
-            return $self->_get_datetime_object;
-        }
-
-        if ($year > $century) { $century-- };
-        if (length $year == 2) { $year = "$century$year" };
-
-        unless (check_date($year, $month, $day)) {
-            $self->_set_failure;
-            $self->_set_error("(invalid date)");
-            return $self->_get_datetime_object;
-        }
-
-        $self->{datetime}->set(
-            year  => $year,
-            month => $month,
-            day   => $day,
-        );
-
-        $self->_set_valid_exp;
-        $self->_set_modified(1);
-
-        if (@{$self->{tokens} || []}) {
-            $self->{count}{tokens}--;
-            $self->_unset_valid_exp;
-            $self->_unset_modified;
-            $self->_process;
+        if ($self->{Prefer_future}) {
+            $self->_advance_future(qw(md));
         }
     }
     else {
-        @{$self->{tokens}} = split ' ', $date_string;
+        @{$self->{tokens}} = split /\s+/, $date_string;
         $self->{data}->__init('tokens')->($self);
-        $self->{count}{tokens} = scalar @{$self->{tokens}};
+        $self->{count}{tokens} = @{$self->{tokens}};
 
         $self->_process;
     }
 
+    my $trace = $self->_trace_string;
+    if (defined $trace) {
+        @{$self->{traces}} = $trace;
+    }
+
     return $self->_get_datetime_object;
 }
 
+sub _params_init
+{
+    my $self = shift;
+    my $params = pop;
+
+    if (@_ > 1) {
+        validate(@_, { string => { type => SCALAR }});
+        my %opts = @_;
+        foreach my $opt (keys %opts) {
+            ${$params->{$opt}} = $opts{$opt};
+        }
+    }
+    else {
+        validate_pos(@_, { type => SCALAR });
+        (${$params->{string}}) = @_;
+    }
+}
+
 sub _parse_init
 {
     my $self = shift;
 
-    if (@_ > 1) {
-        validate(@_, { string => { type => SCALAR }});
-        my %opts             = @_;
-        $self->{Date_string} = $opts{string};
-        (undef)              = $opts{debug}; # legacy
+    $self->_params_init(@_, { string => \$self->{Date_string} });
+
+    my $set_datetime = sub
+    {
+        my ($method, $args) = @_;
+
+        if (exists $self->{Datetime} && $method eq 'now') {
+            $self->{datetime} = dclone($self->{Datetime});
+        }
+        else {
+            $self->{datetime} = DateTime->$method(
+                time_zone => $self->{Time_zone},
+                %$args,
+            );
+        }
+    };
+
+    if ($self->{running_tests}) {
+        $self->{datetime} = $self->{datetime_test}->clone;
     }
     else {
-        validate_pos(@_, { type => SCALAR });
-        ($self->{Date_string}) = @_;
-    }
-
-    unless ($self->{running_tests}) {
-        $self->{datetime} = DateTime->now(time_zone => $self->{Time_zone});
+        $set_datetime->('now', {});
     }
 
     $self->_init_vars;
@@ -217,26 +229,48 @@ sub _parse_init
     $self->_unset_error;
     $self->_unset_valid_exp;
     $self->_unset_trace;
-    $self->_unset_modified;
 }
 
 sub parse_datetime_duration
 {
     my $self = shift;
 
-    $self->_parse_init(@_);
-
+    my $duration_string;
+    $self->_params_init(@_, { string => \$duration_string });
     my $timespan_sep = $self->{data}->__timespan('literal');
 
-    my @date_strings = $self->{Date_string} =~ /$timespan_sep/i
-      ? split /\s+ $timespan_sep \s+/ix, $self->{Date_string}
-      : ($self->{Date_string});
+    my @date_strings = $duration_string =~ /\b $timespan_sep \b/ix
+      ? do { $self->{duration} = true;
+             split /\s+ $timespan_sep \s+/ix, $duration_string }
+      : do { $self->{duration} = false;
+             ($duration_string) };
 
-    my @queue;
+    $self->_pre_duration(\@date_strings);
+    $self->{state} = {};
+
+    my (@queue, @traces);
     foreach my $date_string (@date_strings) {
         push @queue, $self->parse_datetime($date_string);
+        $self->_save_state(
+            valid_expression => $self->_get_valid_exp,
+            failure          => $self->_get_failure,
+            error            => $self->_get_error,
+        );
+        if (@{$self->{traces}}) {
+            push @traces, $self->{traces}[0];
+        }
     }
 
+    $self->_post_duration(\@queue);
+    $self->_restore_state;
+
+    foreach my $member (qw(duration formatted insert state)) {
+        delete $self->{$member};
+    }
+
+    @{$self->{traces}} = @traces;
+    $self->{Input_string} = $duration_string;
+
     return @queue;
 }
 
@@ -253,7 +287,7 @@ sub error
 
     return '' if $self->success;
 
-    my $error  = "'$self->{Date_string}' does not parse ";
+    my $error  = "'$self->{Input_string}' does not parse ";
        $error .= $self->_get_error || '(perhaps you have some garbage?)';
 
     return $error;
@@ -263,15 +297,15 @@ sub trace
 {
     my $self = shift;
 
-    return join "\n", @{$self->{trace}},
-      map  { my $unit = $_; "$unit: $self->{modified}{$unit}" }
-      grep { $_ ne 'total' } keys %{$self->{modified}};
+    return @{$self->{traces}};
 }
 
 sub _process
 {
     my $self = shift;
 
+    my %opts;
+
     if (!exists $self->{lookup}) {
         foreach my $keyword (keys %{$self->{data}->__grammar('')}) {
             push @{$self->{lookup}{scalar @{$self->{data}->__grammar($keyword)->[0]}}}, $keyword;
@@ -279,15 +313,13 @@ sub _process
     }
 
     PARSE: foreach my $keyword (@{$self->{lookup}{$self->{count}{tokens}} || []}) {
-        last if $self->_get_modified >= $self->{count}{tokens};
-
         my @grammar = @{$self->{data}->__grammar($keyword)};
         my $types = shift @grammar;
 
         foreach my $expression (@grammar) {
             my $valid_expression = true;
             my $definition = $expression->[0];
-            my @positions = keys %$definition;
+            my @positions = sort {$a <=> $b} keys %$definition;
             my %regex_stack;
             foreach my $pos (@positions) {
                 if ($types->[$pos] eq 'SCALAR') {
@@ -317,61 +349,110 @@ sub _process
                         "unknown type $types->[$pos]\n";
                 }
             }
+            if ($valid_expression && @{$expression->[2]}) {
+                my $i;
+                foreach my $check (@{$expression->[2]}) {
+                    my @pos = @{$expression->[1][$i++]};
+                    my $error;
+                    $valid_expression &= $check->(\%regex_stack, \@pos, \$error);
+                    unless ($valid_expression) {
+                        $self->_set_error("($error)");
+                        last;
+                    }
+                }
+            }
             if ($valid_expression) {
                 $self->_set_valid_exp;
-                my $i;
-                foreach my $positions (@{$expression->[1]}) {
+                my $i = 0;
+                foreach my $positions (@{$expression->[3]}) {
                     my ($c, @values);
                     foreach my $pos (@$positions) {
-                        $values[$c++] = exists $regex_stack{$pos}
-                          ? $regex_stack{$pos}
-                          : ${$self->_token($pos)};
+                        my $index = ref $pos eq 'HASH' ? (keys %$pos)[0] : $pos;
+                        $values[$c++] = ref $pos
+                          ? $index eq 'VALUE'
+                            ? $pos->{$index}
+                            : $self->SUPER::_helper($pos->{$index}, $regex_stack{$index})
+                          : exists $regex_stack{$index}
+                            ? $regex_stack{$index}
+                            : ${$self->_token($index)};
                     }
-                    @values = map { defined $_ ? $_ : () } @values;
-                    my $meth = 'SUPER::'.$expression->[-1]->[$i++];
-                    $self->$meth(@values);
+                    my $worker = "SUPER::$expression->[5]->[$i]";
+                    $self->$worker(@values, $expression->[4]->[$i++]);
                 }
+                %opts = %{$expression->[6]};
                 last PARSE;
             }
         }
     }
 
-    $self->_post_process_options;
+    $self->_post_process(%opts);
 }
 
-sub _post_process_options
+sub _post_process
 {
     my $self = shift;
+    my %opts = @_;
 
-    if ($self->{Prefer_future}) {
-        my %modified = map { $_ => true } grep { $_ ne 'total' } keys %{$self->{modified}};
+    if (exists $opts{truncate_to}) {
+        $self->{datetime}->truncate(to => $opts{truncate_to});
+    }
 
-        if ($self->{count}{tokens} == 1
-            && (any { $self->{tokens}->[0] =~ /$_/i } @{$self->{data}->{weekdays_all}})
-            && scalar keys %modified == 1
-            && (exists $self->{modified}{day} && $self->{modified}{day} == 1
-            && Day_of_Week($self->{datetime}->year, $self->{datetime}->month, $self->{datetime}->day)
-             < DateTime->now(time_zone => $self->{Time_zone})->wday)
-        ) {
-            $self->{postprocess}{day} = 7;
-        }
-        elsif ((any { my $month = $_; any { $_ =~ /$month/i } @{$self->{tokens}} } @{$self->{data}->{months_all}})
-            && (all { /^(?:day|month)$/ } keys %modified)
-            && (exists $self->{modified}{month} && $self->{modified}{month} == 1)
-            && (exists $self->{modified}{day}
-                  ? $self->{modified}{day} == 1
-                    ? true : false
-                  : true)
-            && ($self->{datetime}->day_of_year < DateTime->now->day_of_year)
-        ) {
-            $self->{postprocess}{year} = 1;
-        }
+    if ($self->{Prefer_future} &&
+        (exists $opts{prefer_future} && $opts{prefer_future})
+    ) {
+        $self->_advance_future;
+    }
+}
+
+sub _advance_future
+{
+    my $self = shift;
+    my %advance = map { $_ => true } @_;
+
+    my %modified = map { $_ => true } keys %{$self->{modified}};
+    my $token_contains = sub
+    {
+        my ($identifier) = @_;
+        return any {
+          my $data = $_;
+          any {
+            my $token = $_;
+            $token =~ /$data/i;
+          } @{$self->{tokens}}
+        } @{$self->{data}->{$identifier}};
+    };
+
+    if ((all { /^(?:second|minute|hour)$/ } keys %modified)
+        && (exists $self->{modified}{hour} && $self->{modified}{hour} == 1)
+        && (exists $self->{modified}{minute} && $self->{modified}{minute} == 1)
+        && $self->{datetime}->hour < DateTime->now(time_zone => $self->{Time_zone})->hour
+    ) {
+        $self->{postprocess}{day} = 1;
+    }
+    elsif ($token_contains->('weekdays_all')
+        && (exists $self->{modified}{day} && $self->{modified}{day} == 1)
+        && ($self->_Day_of_Week($self->{datetime}->year, $self->{datetime}->month, $self->{datetime}->day)
+         < DateTime->now(time_zone => $self->{Time_zone})->wday)
+    ) {
+        $self->{postprocess}{day} = 7;
+    }
+    elsif (($token_contains->('months_all') || $advance{md})
+        && (all { /^(?:day|month)$/ } keys %modified)
+        && (exists $self->{modified}{month} && $self->{modified}{month} == 1)
+        && (exists $self->{modified}{day}
+              ? $self->{modified}{day} == 1
+                ? true : false
+              : true)
+        && ($self->{datetime}->day_of_year < DateTime->now->day_of_year)
+    ) {
+        $self->{postprocess}{year} = 1;
     }
 }
 
 sub _token
 {
-    my ($self, $pos) = @_;
+    my $self = shift;
+    my ($pos) = @_;
 
     my $str = '';
     my $token = $self->{tokens}->[0 + $pos];
@@ -381,7 +462,7 @@ sub _token
       : \$str;
 }
 
-sub _add_trace       { push @{$_[0]->{trace}}, (caller(1))[3] }
+sub _register_trace  { push @{$_[0]->{trace}}, (caller(1))[3] }
 sub _unset_trace     { @{$_[0]->{trace}} = ()                 }
 
 sub _get_error       { $_[0]->{error}         }
@@ -396,35 +477,19 @@ sub _get_valid_exp   { $_[0]->{valid_expression}         }
 sub _set_valid_exp   { $_[0]->{valid_expression} = true  }
 sub _unset_valid_exp { $_[0]->{valid_expression} = false }
 
-sub _get_modified    { $_[0]->{modified}{total} || 0     }
-sub _set_modified    { $_[0]->{modified}{total} += $_[1] }
-sub _unset_modified  { $_[0]->{modified}{total}  = 0     }
-
 sub _get_datetime_object
 {
     my $self = shift;
 
-    $self->{Time_zone} = $self->{datetime}->time_zone->name;
-    $self->{year}      = $self->{datetime}->year;
-    $self->{month}     = $self->{datetime}->month;
-    $self->{day}       = $self->{datetime}->day_of_month;
-    $self->{hour}      = $self->{datetime}->hour;
-    $self->{min}       = $self->{datetime}->minute;
-    $self->{sec}       = $self->{datetime}->second;
-
-    $self->{sec}       = sprintf("%02d", $self->{sec});
-    $self->{min}       = sprintf("%02d", $self->{min});
-    $self->{hour}      = sprintf("%02d", $self->{hour});
-    $self->{day}       = sprintf("%02d", $self->{day});
-    $self->{month}     = sprintf("%02d", $self->{month});
-
-    my $dt = DateTime->new(time_zone => $self->{Time_zone},
-                           year      => $self->{year},
-                           month     => $self->{month},
-                           day       => $self->{day},
-                           hour      => $self->{hour},
-                           minute    => $self->{min},
-                           second    => $self->{sec});
+    my $dt = DateTime->new(
+        time_zone => $self->{datetime}->time_zone->name,
+        year      => $self->{datetime}->year,
+        month     => $self->{datetime}->month,
+        day       => $self->{datetime}->day_of_month,
+        hour      => $self->{datetime}->hour,
+        minute    => $self->{datetime}->minute,
+        second    => $self->{datetime}->second,
+    );
 
     foreach my $unit (keys %{$self->{postprocess}}) {
         $dt->add("${unit}s" => $self->{postprocess}{$unit});
@@ -436,15 +501,13 @@ sub _get_datetime_object
 # solely for testing purpose
 sub _set_datetime
 {
-    my ($self, $year, $month, $day, $hour, $min, $sec, $tz) = @_;
+    my $self = shift;
+    my ($time, $tz) = @_;
 
-    $self->{datetime} = DateTime->new(time_zone => $tz || 'floating',
-                                      year      => $year,
-                                      month     => $month,
-                                      day       => $day,
-                                      hour      => $hour,
-                                      minute    => $min,
-                                      second    => $sec);
+    $self->{datetime_test} = DateTime->new(
+        time_zone => $tz || 'floating',
+        %$time,
+    );
     $self->{running_tests} = true;
 }
 
@@ -476,6 +539,8 @@ DateTime::Format::Natural - Create machine readable date/time with natural parsi
      warn $parser->error;
  }
 
+ @traces = $parser->trace;
+
 =head1 DESCRIPTION
 
 C<DateTime::Format::Natural> takes a string with a human readable date/time and creates a
@@ -489,9 +554,10 @@ Creates a new C<DateTime::Format::Natural> object. Arguments to C<new()> are opt
 not necessarily required.
 
  $parser = DateTime::Format::Natural->new(
+           datetime      => DateTime->new(...),
            lang          => 'en',
            format        => 'mm/dd/yy',
-           prefer_future => '[0|1]'
+           prefer_future => '[0|1]',
            time_zone     => 'floating',
            daytime       => { morning   => 06,
                               afternoon => 13,
@@ -501,6 +567,10 @@ not necessarily required.
 
 =over 4
 
+=item * C<datetime>
+
+Overrides the present now with a L<DateTime> object provided.
+
 =item * C<lang>
 
 Contains the language selected, currently limited to C<en> (english).
@@ -512,7 +582,7 @@ Specifies the format of numeric dates, defaults to 'C<d/m/y>'.
 
 =item * C<prefer_future>
 
-Turns ambiguous weekdays/months to their futuristic relatives. Accepts a boolean,
+Turns ambiguous weekdays/months to their future relatives. Accepts a boolean,
 defaults to false.
 
 =item * C<time_zone>
@@ -522,8 +592,8 @@ recognized by L<DateTime>. Defaults to 'floating'.
 
 =item * C<daytime>
 
-A hash reference consisting of customized daytime hours, which may be
-selectively changed.
+An anonymous hash reference consisting of customized daytime hours,
+which may be selectively changed.
 
 =back
 
@@ -531,7 +601,7 @@ selectively changed.
 
 =head2 parse_datetime
 
-Creates a C<DateTime> object from a human readable date/time string.
+Returns a L<DateTime> object constructed from a human readable date/time string.
 
  $dt = $parser->parse_datetime($date_string);
  $dt = $parser->parse_datetime(string => $date_string);
@@ -544,13 +614,12 @@ The date string.
 
 =back
 
-Returns a L<DateTime> object.
-
 =head2 parse_datetime_duration
 
-Creates one or more C<DateTime> object(s) from a human readable date/time string
-which may contain timespans/durations. 'Same' interface & options as C<parse_datetime()>,
-but must be explicitly called in list context.
+Returns one or more L<DateTime> object(s) constructed from a human readable
+date/time string which may contain timespans/durations. I<Same> interface
+and options as C<parse_datetime()>, but should be explicitly called in
+list context.
 
  @dt = $parser->parse_datetime_duration($date_string);
  @dt = $parser->parse_datetime_duration(string => $date_string);
@@ -566,8 +635,9 @@ Returns the error message if the parsing did not succeed.
 
 =head2 trace
 
-Returns a trace of methods which were called within the Base class and
-a summary how often certain units have been modified.
+Returns one or more strings with traces of methods which were called within
+the Base class and a summary how often certain units have been modified.
+More than one string is commonly returned for durations.
 
 =head1 GRAMMAR
 
@@ -580,7 +650,19 @@ you're intending to hack a bit on the grammar guts.
 =head1 EXAMPLES
 
 See the classes C<DateTime::Format::Natural::Lang::[language_code]> for a
-overview of current valid input.
+overview of currently valid input.
+
+=head1 BUGS & CAVEATS
+
+C<parse_datetime()>/C<parse_datetime_duration()> always return one or more
+DateTime objects regardless whether the parse was successful or not. In
+case no valid expression was found or a failure occurred, an unaltered
+DateTime object with its initial values (most often the "current" now) is
+likely to be returned. It is therefore recommended to use C<success()> to
+assert that the parse did succeed (at least, for common uses), otherwise
+the absence of a parse failure cannot be guaranteed.
+
+C<parse_datetime()> is not capable of handling durations.
 
 =head1 CREDITS
 
@@ -588,7 +670,7 @@ Thanks to Tatsuhiko Miyagawa for the initial inspiration. See Miyagawa's journal
 entry L<http://use.perl.org/~miyagawa/journal/31378> for more information.
 
 Furthermore, thanks to (in order of appearance) who have contributed
-valuable suggestions & patches:
+valuable suggestions and patches:
 
  Clayton L. Scott
  Dave Rolsky
@@ -607,6 +689,12 @@ valuable suggestions & patches:
  Pat Kale
  Ankur Gupta
  Alex Bowley
+ Elliot Shank
+ Anirvan Chatterjee
+ Michael Reddick
+ Christian Brink
+ Giovanni Pensa
+ Andrew Sterling Hanenkamp
 
 =head1 SEE ALSO
 
@@ -621,6 +709,6 @@ Steven Schubiger <schubiger@cpan.org>
 This program is free software; you may redistribute it and/or
 modify it under the same terms as Perl itself.
 
-See L<http://www.perl.com/perl/misc/Artistic.html>
+See L<http://dev.perl.org/licenses/>
 
 =cut
